/*
 * Copyright Rasmus Torkel 2013
 * Author    Rasmus Torkel 2013
 * 
 * You may only use this software subject to the terms and conditions which either you or your
 * organisation would have downloaded with this software and which should have been placed
 * somewhere accessible to you. These conditions are likely to
 * -- allow you or your organisation to determine whether you or your organisation
 *    may use this software for free or not and if not how to pay and how much;
 * -- prohibit or restrict republishing of this software and transmission to other organisations,
 *    even related companies within the same group;
 * -- prohibit the renaming of this class and the containing package.
 *    
 * Neither you nor your organisation may modify this header. However, you or your organisation
 * may modify anything below this header. If so, you or your organisation must place a comment
 * to that effect immediately below this header.
 */
package rasmus_torkel.config.exception;

import java.io.File;
import java.io.IOException;

import rasmus_torkel.config.ConfigInvokeStack;
import rasmus_torkel.text.read.exception.ReadIoException;

/**
 * Exception to do with IO while reading configuration.
 * 
 * @author Rasmus Torkel
 *
 */
public class ConfigIoException extends ReadIoException
{
    public
    ConfigIoException(String      message,
                      IOException e)
    {
        super(message, e);
    }

    public
    ConfigIoException(String      activityClause,
                      File        file,
                      IOException e)
    {
        super(makeMessage(activityClause, file, e));
    }

    public
    ConfigIoException(String            activityClause,
                      File              file,
                      ConfigInvokeStack invokeStack,
                      IOException       e)
    {
        super(makeMessage(activityClause, file, invokeStack, e));
    }

    private static String
    makeMessage(String      activityClause,
                File        file,
                IOException e)
    {
        String fileName;
        try
        {
            fileName = file.getCanonicalPath();
        }
        catch (IOException e1)
        {
            fileName = file.toString();
        }
        return activityClause + " " + fileName + " led to " + e;
    }
    
    private static String
    makeMessage(String            activityClause,
                File              file,
                ConfigInvokeStack invokeStack,
                IOException       e)
    {

        String fileName;
        try
        {
            fileName = file.getCanonicalPath();
        }
        catch (IOException e1)
        {
            fileName = file.toString();
        }
        return activityClause + " " + fileName + " led to " + e;
    }
}
