package rasmus_torkel.misc;

import java.lang.reflect.Field;
import java.util.HashMap;


public class ArrayMakerCache
{
    public static final ArrayMakerCache INSTANCE = new ArrayMakerCache();
    
    private HashMap<String,ArrayMaker<?>> _classNameToArrayMaker = new HashMap<String, ArrayMaker<?>>();
    
    public class ArrayMakerCacheException extends RuntimeException
    {
        public
        ArrayMakerCacheException(String message)
        {
            super(message);
        }
    }
    
    public synchronized ArrayMaker<?>
    getArrayMaker(Class<?> type)
    {
        if (type == null)
        {
            return null;
        }
        String className = type.getName();
        ArrayMaker<?> factory = _classNameToArrayMaker.get(className);
        if (factory != null)
        {
            return factory;
        }
        Field field;
        try
        {
            field = type.getField("ARRAY_MAKER");
        }
        catch (NoSuchFieldException e)
        {
            throw new ArrayMakerCacheException("Class " + className + " does not have a ARRAY_MAKER field");
        }
        catch (SecurityException e)
        {
            throw new ArrayMakerCacheException("Class " + className + " has ARRAY_MAKER field which is not accessible");
        }
        Object factoryObj;
        try
        {
            factoryObj = field.get(null);
        }
        catch (IllegalArgumentException e)
        {
            throw new ArrayMakerCacheException("Class " + className + " resulted in an unexpected exception for Field.get: " + e);
        }
        catch (IllegalAccessException e)
        {
            throw new ArrayMakerCacheException("Class " + className + " has ARRAY_MAKER field which is not accessible");
        }
        catch (NullPointerException e)
        {
            throw new ArrayMakerCacheException("Class " + className + " has ARRAY_MAKER field which is not static");
        }
        if (factoryObj == null)
        {
            throw new ArrayMakerCacheException("Class " + className + " has ARRAY_MAKER field which is null");
        }
        if (factoryObj instanceof ArrayMaker)
        {
            factory = (ArrayMaker<?>)factoryObj;
        }
        else
        {
            throw new ArrayMakerCacheException(
                    "Class " + className + " has ARRAY_MAKER field which is of class " + factoryObj.getClass().getName() +
                    " which not a subclass of " + ArrayMaker.class.getName());
        }
        _classNameToArrayMaker.put(className, factory);
        return factory;
    }
}
