package rasmus_torkel.misc;

public class ClassUtil
{
    /**
     * Gets the name of a class. Why not just call Class.getName()?
     * Because that method does not work for array classes.
     * @param clas The class for which we want the name
     * @return the name of the class
     */
    public static String
    classToName(Class<?> clas)
    {
        if (clas == null)
        {
            throw new RuntimeException("class is null");
        }
        else if (clas.isArray())
        {
            Class<?> elementClass = clas.getComponentType();
            String elementClassName = classToName(elementClass);
            return elementClassName + "[]";
        }
        else
        {
            return clas.getName();
        }
    }
    
    public static String
    ultimateComponentClassName(String className)
    {
        return ultimateComponentClassName(className, className.length());
    }
    
    private static String
    ultimateComponentClassName(String className,
                               int    classNameLength)
    {
        if (classNameLength < 2)
        {
            return className.substring(0, classNameLength);
        }
        else if (   className.charAt(classNameLength - 2) == '['
                 && className.charAt(classNameLength - 1) == ']')
        {
            return ultimateComponentClassName(className, classNameLength - 2);
        }
        else
        {
            return className.substring(0, classNameLength);
        }
    }
    
    public static int
    dimensionQty(String className)
    {
        return dimensionQty(className, className.length());
    }
    
    private static int
    dimensionQty(String className,
                 int    classNameLength)
    {
        if (classNameLength < 2)
        {
            return 0;
        }
        else if (   className.charAt(classNameLength - 2) == '['
                 && className.charAt(classNameLength - 1) == ']')
        {
            return 1 + dimensionQty(className, classNameLength - 2);
        }
        else
        {
            return 0;
        }
    }
    /*
    public static int
    dimensionQty(Class<?> clas)
    {
        if (clas == null)
        {
            throw new RuntimeException("class is null");
        }
        else if (clas.isArray())
        {
            Class<?> elementClass = clas.getComponentType();
            return 1 + dimensionQty(elementClass);
        }
        else
        {
            return 0;
        }
    }*/
}
