/*
 * Copyright Rasmus Torkel 2015, 2016
 * Author    Rasmus Torkel 2015, 2016
 * 
 * You may only use this software subject to the terms and conditions which either you or your
 * organisation would have downloaded with this software and which should have been placed
 * somewhere accessible to you. These conditions are likely to
 * -- allow you or your organisation to determine whether you or your organisation
 *    may use this software for free or not and if not how to pay and how much;
 * -- prohibit or restrict republishing of this software and transmission to other organisations,
 *    even related companies within the same group;
 * -- prohibit the renaming of this class and the containing package.
 *    
 * Neither you nor your organisation may modify this header. However, you or your organisation
 * may modify anything below this header. If so, you or your organisation must place a comment
 * to that effect immediately below this header.
 */
package rasmus_torkel.misc;

public class IntUtil
{
    public static String
    toHexUpper(int n)
    {
        return toHex(n, _hexUpperChars);
    }
    
    private static String
    toHex(int           n,
          char[]        hexChars)
    {
        char[] chars = new char[8];
        for (int i = 0; i < 8; i++)
        {
            int smallInt = (n & _masks[i]) >>> _rightShifts[i];
            chars[i] = hexChars[smallInt];
        }
        return new String(chars);
    }
    
    private static final int[] _masks = { 0xF0000000, 0xF000000, 0xF00000, 0xF0000, 0xF000, 0xF00, 0xF0, 0xF };
    private static final int[] _rightShifts = { 28, 24, 20, 16, 12, 8, 4, 0 };
    private static final char[] _hexUpperChars = { '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'A', 'B', 'C', 'D', 'E', 'F' };
    
    public static String
    zeroPad(int n,
            int desiredLength)
    {
        StringBuilder buf = new StringBuilder();
        zeroPad(buf, n, desiredLength);
        return buf.toString();
    }
    
    public static void
    zeroPad(StringBuilder buf,
            int           n,
            int           desiredLength)
    {
        if (n < 0)
        {
            buf.append('-');
            zeroPad(buf, -n, desiredLength - 1);
        }
        else
        {
            int length = intLength(n);
            for (int i = length; i < desiredLength; i++)
            {
                buf.append('0');
            }
            buf.append(n);
        }
    }
    
    private static int highest10Power = 1000000000;
    private static int highestPositiveIntLength = 10;
    
    public static int
    intLength(int n)
    {
        if (n == 0)
        {
            return 1;
        }
        if (n < 0)
        {
            return 1 + intLength(-n);
        }
        int lengthAttempt = highestPositiveIntLength;
        int thresholdAttempt = highest10Power;
        while (n < thresholdAttempt)
        {
            thresholdAttempt /= 10;
            lengthAttempt -= 1;
        }
        return lengthAttempt;
    }
    
    public static void
    join(StringBuilder   buf,
         int[]           joinees,
         JoinPunctuation punctuation)
    {
        join(buf, joinees, punctuation._start, punctuation._separator, punctuation._end);
    }
    
    public static void
    join(StringBuilder buf,
         int[]         joinees,
         String        start,
         String        sep,
         String        end)
    {
        if (start != null)
        {
            buf.append(start);
        }
        join(buf, joinees, sep);
        if (end != null)
        {
            buf.append(end);
        }
    }
    
    public static void
    join(StringBuilder buf,
         int[]         joinees,
         String        sep)
    {
        for (int i = 0; i < joinees.length; i++)
        {
            if (i > 0 && sep != null)
            {
                buf.append(sep);
            }
            buf.append(joinees[i]);
        }
    }
}
