/*
 * Copyright Rasmus Torkel 2016
 * Author    Rasmus Torkel 2016
 * 
 * You may only use this software subject to the terms and conditions which either you or your
 * organisation would have downloaded with this software and which should have been placed
 * somewhere accessible to you. These conditions are likely to
 * -- allow you or your organisation to determine whether you or your organisation
 *    may use this software for free or not and if not how to pay and how much;
 * -- prohibit or restrict republishing of this software and transmission to other organisations,
 *    even related companies within the same group;
 * -- prohibit the renaming of this class and the containing package.
 *    
 * Neither you nor your organisation may modify this header. However, you or your organisation
 * may modify anything below this header. If so, you or your organisation must place a comment
 * to that effect immediately below this header.
 */
package rasmus_torkel.misc;

public class ObjectUtil
{
    /**
     * Like the standard equals function except that two null objects are
     * equal and a non-null object is never equal to a null object.
     * @param o1 First object in comparison
     * @param o2 Second object in comparison
     * @return
     */
    public static boolean
    areEqual(Object o1,
             Object o2)
    {
        if (o1 == null && o2 == null)
        {
            return true;
        }
        else if (o1 == null || o2 == null)
        {
            return false;
        }
        else
        {
            return o1.equals(o2);
        }
    }
    
    public static String
    join(Object[]        joinees,
         JoinPunctuation punctuation)
    {
        StringBuilder buf = new StringBuilder();
        join(buf, joinees, punctuation);
        return buf.toString();
    }
    
    public static void
    join(StringBuilder   buf,
         Object[]        joinees,
         JoinPunctuation punctuation)
    {
        join(buf, joinees, punctuation._start, punctuation._separator, punctuation._end);
    }
    
    public static void
    join(StringBuilder buf,
         Object[]      joinees,
         String        start,
         String        sep,
         String        end)
    {
        if (start != null)
        {
            buf.append(start);
        }
        join(buf, joinees, sep);
        if (end != null)
        {
            buf.append(end);
        }
    }
    
    public static void
    join(StringBuilder buf,
         Object[]      joinees,
         String        sep)
    {
        if (joinees == null)
        {
            return;
        }
        for (int i = 0; i < joinees.length; i++)
        {
            if (i > 0 && sep != null)
            {
                buf.append(sep);
            }
            buf.append(joinees[i]);
        }
    }
    
    public static String
    join2D(Object[][]      joinees2D,
           JoinPunctuation punctuation)
    {
        StringBuilder buf = new StringBuilder();
        join2D(buf, joinees2D, punctuation);
        return buf.toString();
    }
    
    public static void
    join2D(StringBuilder   buf,
           Object[][]      joinees2D,
           JoinPunctuation punctuation)
    {
        join2D(buf, joinees2D, punctuation, punctuation);
    }
    
    public static void
    join2D(StringBuilder   buf,
           Object[][]      joinees2D,
           JoinPunctuation punctuation2D,
           JoinPunctuation punctuation1D)
    {
        if (joinees2D == null)
        {
            return;
        }
        if (punctuation2D._start != null)
        {
            buf.append(punctuation2D._start);
        }
        for (int i = 0; i < joinees2D.length; i++)
        {
            if (i > 0 && punctuation2D._separator != null)
            {
                buf.append(punctuation2D._separator);
            }
            join(buf, joinees2D[i], punctuation1D);
        }
        if (punctuation2D._end != null)
        {
            buf.append(punctuation2D._end);
        }
    }
    
    public static String
    join3D(Object[][]      joinees2D,
           JoinPunctuation punctuation)
    {
        StringBuilder buf = new StringBuilder();
        join2D(buf, joinees2D, punctuation);
        return buf.toString();
    }
    
    public static void
    join3D(StringBuilder   buf,
           Object[][][]    joinees3D,
           JoinPunctuation punctuation)
    {
        join3D(buf, joinees3D, punctuation, punctuation, punctuation);
    }
    
    public static void
    join3D(StringBuilder   buf,
           Object[][][]    joinees3D,
           JoinPunctuation punctuation3D,
           JoinPunctuation punctuation2D,
           JoinPunctuation punctuation1D)
    {
        if (joinees3D == null)
        {
            return;
        }
        if (punctuation3D._start != null)
        {
            buf.append(punctuation2D._start);
        }
        for (int i = 0; i < joinees3D.length; i++)
        {
            if (i > 0 && punctuation3D._separator != null)
            {
                buf.append(punctuation3D._separator);
            }
            join2D(buf, joinees3D[i], punctuation2D);
        }
        if (punctuation3D._end != null)
        {
            buf.append(punctuation2D._end);
        }
    }
}
