/*
 * Copyright Rasmus Torkel 2016
 * Author    Rasmus Torkel 2016
 * 
 * You may only use this software subject to the terms and conditions which either you or your
 * organisation would have downloaded with this software and which should have been placed
 * somewhere accessible to you. These conditions are likely to
 * -- allow you or your organisation to determine whether you or your organisation
 *    may use this software for free or not and if not how to pay and how much;
 * -- prohibit or restrict republishing of this software and transmission to other organisations,
 *    even related companies within the same group;
 * -- prohibit the renaming of this class and the containing package.
 *    
 * Neither you nor your organisation may modify this header. However, you or your organisation
 * may modify anything below this header. If so, you or your organisation must place a comment
 * to that effect immediately below this header.
 */
package rasmus_torkel.misc;

import java.util.List;

public class StringUtil
{
    public static final ArrayMaker3D<String> ARRAY_MAKER = new ArrayMaker3D<String>()
    {
        
        @Override
        public String[] newArray(int size)
        {
            return new String[size];
        }
        
        @Override
        public String[][] newArray2D(int size)
        {
            return new String[size][];
        }
        
        @Override
        public String[][][] newArray3D(int size)
        {
            return new String[size][][];
        }
    };
    
    public static boolean
    isPlainPrintable(String s)
    {
        if (s == null)
        {
            return false;
        }
        int length = s.length();
        for (int i = 0; i < length; i++)
        {
            char ch = s.charAt(i);
            if (!CharUtil.isPlainPrintable(ch))
            {
                return false;
            }
        }
        return true;
    }
    
    /**
     * Makes a plain-printable phrase of a plain-printable name and a
     * value which may not be plain-printable.
     * @param name   It is a precondition that this name is plain printable
     * @param value  Value which may or may not be plain printable. If it
     *               is not plain printable, it may not be included.
     * @return
     */
    public static String
    namedValueToPlainPrintablePhrase(String name,
                                     String value)
    {
        if (isPlainPrintable(value))
        {
            return name + " " + value;
        }
        else
        {
            return "non-plain-printable " + name;
        }
    }
    /* untested
    public static String
    frontPad(String s,
             char   padChar,
             int    desiredLength)
    {
        int sLength = s.length();
        if (sLength >= 0)
        {
            return s;
        }
        StringBuilder buf = new StringBuilder(desiredLength);
        for (int i = sLength; i < desiredLength; i++)
        {
            buf.append(padChar);
        }
        buf.append(s);
        return s;
    }*/
    
    public static String[]
    listToArray(List<String> stringList)
    {
        String[] strings = new String[stringList.size()];
        stringList.toArray(strings);
        return strings;
    }
    
    public static void
    join(StringBuilder     buf,
         String[]          joinees,
         JoinPunctuation punctuation)
    {
        join(buf, joinees, punctuation._start, punctuation._separator, punctuation._end);
    }
    
    public static void
    join(StringBuilder buf,
         String[]      joinees,
         String        start,
         String        sep,
         String        end)
    {
        if (start != null)
        {
            buf.append(start);
        }
        join(buf, joinees, sep);
        if (end != null)
        {
            buf.append(end);
        }
    }
    
    public static void
    join(StringBuilder buf,
         String[]      joinees,
         String        sep)
    {
        for (int i = 0; i < joinees.length; i++)
        {
            if (i > 0 && sep != null)
            {
                buf.append(sep);
            }
            buf.append(joinees[i]);
        }
    }
}
