/*
 * Copyright Rasmus Torkel 2013
 * Author    Rasmus Torkel 2012/2013
 * 
 * You may only use this software subject to the terms and conditions which either you or your
 * organisation would have downloaded with this software and which should have been placed
 * somewhere accessible to you. These conditions are likely to
 * -- allow you or your organisation to determine whether you or your organisation
 *    may use this software for free or not and if not how to pay and how much;
 * -- prohibit or restrict republishing of this software and transmission to other organisations,
 *    even related companies within the same group;
 * -- prohibit the renaming of this class and the containing package.
 *    
 * Neither you nor your organisation may modify this header. However, you or your organisation
 * may modify anything below this header. If so, you or your organisation must place a comment
 * to that effect immediately below this header.
 */
package rasmus_torkel.set.chars;

/**
 * This type of set contains an array if individual characters.
 * 
 * @author Rasmus Torkel
 *
 */
public class ArrayCharSet extends CharSet
{
    private final char[] _chars;

    /**
     * Constructor.
     * 
     * @param chars The characters in the set
     * @param label Populates {@link #_label} field.
     */
    public ArrayCharSet(char[] chars, String label)
    {
        super(max(chars), label);
        _chars = chars;
    }
    
    private static char
    max(char[] chars)
    {
        char max = Character.MIN_VALUE;
        for (char ch : chars)
        {
            if (ch > max)
            {
                max = ch;
            }
        }
        return max;
    }
    
    @Override
    public boolean
    isIn(char ch)
    {
        for (char keptCh : _chars)
        {
            if (ch == keptCh)
            {
                return true;
            }
        }
        return false;
    }
}
