/*
 * Copyright Rasmus Torkel 2013
 * Author    Rasmus Torkel 2012/2013
 * 
 * You may only use this software subject to the terms and conditions which either you or your
 * organisation would have downloaded with this software and which should have been placed
 * somewhere accessible to you. These conditions are likely to
 * -- allow you or your organisation to determine whether you or your organisation
 *    may use this software for free or not and if not how to pay and how much;
 * -- prohibit or restrict republishing of this software and transmission to other organisations,
 *    even related companies within the same group;
 * -- prohibit the renaming of this class and the containing package.
 *    
 * Neither you nor your organisation may modify this header. However, you or your organisation
 * may modify anything below this header. If so, you or your organisation must place a comment
 * to that effect immediately below this header.
 */
package rasmus_torkel.set.chars;

/**
 * A char set consisting of other char sets.
 * 
 * @author Rasmus Torkel
 *
 */
public class ComboCharSet extends CharSet
{
    private final CharSet[] _sets;
    
    /**
     * Constructor.
     * @param label Populates {@link #_label} field.
     * @param sets  The subsets of which this set is to consist.
     */
    public
    ComboCharSet(String     label,
                 CharSet... sets)
    {
        super(max(sets), label);
        _sets = sets;
    }
    
    private static char
    max(CharSet[] sets)
    {
        char max = Character.MIN_VALUE;
        for (CharSet keptSet : sets)
        {
            if (keptSet._max > max)
            {
                max = keptSet._max;
            }
        }
        return max;
    }

    @Override
    public boolean
    isIn(char ch)
    {
        for (CharSet set : _sets)
        {
            if (set.isIn(ch))
            {
                return true;
            }
        }
        return false;
    }
}
