/*
 * Copyright Rasmus Torkel 2013
 * Author    Rasmus Torkel 2012/2013
 * 
 * You may only use this software subject to the terms and conditions which either you or your
 * organisation would have downloaded with this software and which should have been placed
 * somewhere accessible to you. These conditions are likely to
 * -- allow you or your organisation to determine whether you or your organisation
 *    may use this software for free or not and if not how to pay and how much;
 * -- prohibit or restrict republishing of this software and transmission to other organisations,
 *    even related companies within the same group;
 * -- prohibit the renaming of this class and the containing package.
 *    
 * Neither you nor your organisation may modify this header. However, you or your organisation
 * may modify anything below this header. If so, you or your organisation must place a comment
 * to that effect immediately below this header.
 */
package rasmus_torkel.set.chars;

/**
 * This interface contains just a whole bunch of predefined character sets.
 * 
 * @author Torkel
 *
 */
public interface DefinedCharSets
{
    public static final RangeCharSet BITS_7 = new RangeCharSet((char)0, (char)127, "7_Bits");
    public static final RangeCharSet BITS_8 = new RangeCharSet((char)0, (char)255, "8_Bits");
    public static final RangeCharSet BITS_16 = new RangeCharSet((char)0, Character.MAX_VALUE, "16_Bits");
    
    public static final RangeCharSet BITS_7_NON_NULL = new RangeCharSet((char)1, BITS_7._max, "7_Bits_Non_Null");
    public static final RangeCharSet BITS_8_NON_NULL = new RangeCharSet((char)1, BITS_8._max, "8_Bits_Non_Null");
    public static final RangeCharSet BITS_16_NON_NULL = new RangeCharSet((char)1, BITS_16._max, "16_Bits_Non_Null");
    
    public static final SingleCharSet NULL_CHAR = new SingleCharSet((char)0, "Null");
    public static final SingleCharSet HORIZONTAL_TAB = new SingleCharSet('\t', "Horizontal_Tab");
    public static final SingleCharSet LINE_FEED = new SingleCharSet('\n', "Line_Feed");
    public static final SingleCharSet CARRIAGE_RETURN = new SingleCharSet('\r', "Carriage_Return");
    public static final SingleCharSet UNDERSCORE = new SingleCharSet('_', "Underscore");
    public static final SingleCharSet DOT = new SingleCharSet('.', "Dot");
    
    public static final RangeCharSet ENGLISH_KEYBOARD_SYMBOLS =
        new RangeCharSet((char)33, (char)126, "English_Keyboard_Symbols");
    
    public static final RangeCharSet ENGLISH_KEYBOARD_NON_CONTROL =
        new RangeCharSet((char)32, ENGLISH_KEYBOARD_SYMBOLS._max, "English_Keyboard_Non_Control_Chars");
    
    public static final ComboCharSet ENGLISH_KEYBOARD_UNIX_TEXT =
        new ComboCharSet("English_Keyboard_Unix_Text", HORIZONTAL_TAB, LINE_FEED, ENGLISH_KEYBOARD_NON_CONTROL);
    
    public static final ComboCharSet ENGLISH_KEYBOARD_GENERAL_TEXT =
        new ComboCharSet("English_Keyboard_General_Text", HORIZONTAL_TAB, LINE_FEED, CARRIAGE_RETURN, ENGLISH_KEYBOARD_NON_CONTROL);
    
    public static final RangeCharSet DIGITS = new RangeCharSet('0', '9', "Digits");
    
    public static final RangeCharSet LATIN_UPPER_CASE_LETTERS =
        new RangeCharSet('A', 'Z', "Latin_Upper_Case_Letters");
    
    public static final RangeCharSet LATIN_LOWER_CASE_LETTERS =
        new RangeCharSet('a', 'z', "Latin_Lower_Case_Letters");
    
    public static final ComboCharSet LATIN_LETTERS =
        new ComboCharSet("Latin_Letters", LATIN_UPPER_CASE_LETTERS, LATIN_LOWER_CASE_LETTERS);
    
    public static final ComboCharSet VARIABLE_FIRST_CHARS =
        new ComboCharSet("Variable_First_Chars", LATIN_UPPER_CASE_LETTERS, UNDERSCORE, LATIN_LOWER_CASE_LETTERS);
    
    public static final ComboCharSet VARIABLE_CHARS =
        new ComboCharSet("Variable_Chars", DIGITS, LATIN_UPPER_CASE_LETTERS, UNDERSCORE, LATIN_LOWER_CASE_LETTERS);
    
    public static final ComboCharSet CLASS_NAME_CHARS =
        new ComboCharSet("Variable_Chars", DIGITS, LATIN_UPPER_CASE_LETTERS, UNDERSCORE, LATIN_LOWER_CASE_LETTERS, DOT);
}
