/*
 * Copyright Rasmus Torkel 2012
 * Author    Rasmus Torkel 2012
 * 
 * You may only use this software subject to the terms and conditions which either you or your
 * organisation would have downloaded with this software and which should have been placed
 * somewhere accessible to you. These conditions are likely to
 * -- allow you or your organisation to determine whether you or your organisation
 *    may use this software for free or not and if not how to pay and how much;
 * -- prohibit or restrict republishing of this software and transmission to other organisations,
 *    even related companies within the same group;
 * -- prohibit the renaming of this class and the containing package.
 *    
 * Neither you nor your organisation may modify this header. However, you or your organisation
 * may modify anything below this header. If so, you or your organisation must place a comment
 * to that effect immediately below this header.
 */
package rasmus_torkel.test_framework;

public class UnitTest
{
    public static void
    printArgs(String[] args)
    {
        System.out.println("Quantity of arguments: " + args.length);
        for (int i = 0; i < args.length; i++)
        {
            System.out.println(i + ": " + args[i]);
        }
        System.out.println("");
    }
    
    public static void
    announceTestClassStart(Class<?> testClass)
    {
        System.out.println("");
        System.out.println("");
        System.out.println("********************************************************************************");
        System.out.println("Unit Test " + testClass.getName() + " starting");
        System.out.println("********************************************************************************");
        System.out.println("");
    }
    
    public static void
    printTestPartSeparator()
    {
        System.out.println("--------------------------------------------------------------------------------");
    }
    
    public static void
    printEndTest()
    {
        System.out.println("********************************************************************************");
        System.out.println("");
    }
    
    /**
     * Prints ruler to match the width of the supplied string. By width we mean the length of
     * the longest line within it, not counting the line terminators.
     * @param s
     */
    public static void
    printRuler(String s)
    {
        int width = width(s, 4);
        printRuler(width);
    }
    
    /**
     * Prints ruler to match the width of the supplied string. By width we mean the length of
     * the longest line within it, not counting the line terminators.
     * @param s        String for which we want tab width
     * @param tabWidth Most characters are one unit wide. Tabs are the exception.
     */
    public static void
    printRuler(String s,
               int    tabWidth)
    {
        int width = width(s, tabWidth);
        printRuler(width);
    }
    
    public static int
    width(String s,
          int    tabWidth)
    {
        int sLength = s.length();
        int maxWidth = 0;
        int currentWidth = 0;
        for (int i = 0; i < sLength; i++)
        {
            char ch = s.charAt(i);
            if (ch == '\t')
            {
                int tabMod = currentWidth % tabWidth;
                currentWidth += tabWidth - tabMod;
            }
            else if (ch == '\n' || ch == '\r')
            {
                if (currentWidth > maxWidth)
                {
                    maxWidth = currentWidth;
                }
                currentWidth = 0;
            }
            else
            {
                currentWidth++;
            }
        }
        if (currentWidth > maxWidth)
        {
            maxWidth = currentWidth;
        }
        return maxWidth;
    }
    
    public static void
    printRuler(int length)
    {
        int oddLength = length % 10;
        int roundedUpLength;
        if (oddLength > 0)
        {
            roundedUpLength = length + (10 - oddLength);
        }
        else
        {
            roundedUpLength = length;
        }
        StringBuilder buf = new StringBuilder(roundedUpLength);
        for (int i = 10; i <= roundedUpLength; i += 10)
        {
            if (i <= 10)
            {
                buf.append("1234567810");
            }
            else if (i < 100)
            {
                buf.append(" 2345678");
                buf.append(i);
            }
            else if (i < 1000)
            {
                buf.append(" 234567");
                buf.append(i);
            }
            else if (i < 10000)
            {
                buf.append(" 23456");
                buf.append(i);
            }
            else
            {
                throw new RuntimeException(length + " is too big for ruler length");
            }
        }
        System.out.println(buf.toString());
    }
    
    public static void
    announceTestClassPass(Class<?> testClass)
    {
        System.out.println("");
        System.out.println("");
        //System.out.println("********************************************************************************");
        System.out.println("Unit Test " + testClass.getName() + " passed");
        System.out.println("");
    }
    
    public void
    announceTestClassStart()
    {
        announceTestClassStart(getClass());
    }
    
    public void
    announceTestClassPass()
    {
        announceTestClassPass(getClass());
    }
    
    public UnitTestError
    fail(String message)
    {
        System.out.println(message);
        throw new UnitTestError(getClass());
    }
    
    /**
     * Test for functionality within this class
     * @param args  Not used
     */
    public static void
    main(String[] args)
    {
        printRuler(0);
        printRuler(1);
        printRuler(9);
        printRuler(10);
        printRuler(11);
        printRuler(19);
        printRuler(20);
        printRuler(21);
        printRuler(99);
        printRuler(100);
        printRuler(101);
        printRuler(999);
        printRuler(1000);
        printRuler(1001);
    }
}
