/*
 * Copyright Rasmus Torkel 2012
 * Author    Rasmus Torkel 2012
 * 
 * You may only use this software subject to the terms and conditions which either you or your
 * organisation would have downloaded with this software and which should have been placed
 * somewhere accessible to you. These conditions are likely to
 * -- allow you or your organisation to determine whether you or your organisation
 *    may use this software for free or not and if not how to pay and how much;
 * -- prohibit or restrict republishing of this software and transmission to other organisations,
 *    even related companies within the same group;
 * -- prohibit the renaming of this class and the containing package.
 *    
 * Neither you nor your organisation may modify this header. However, you or your organisation
 * may modify anything below this header. If so, you or your organisation must place a comment
 * to that effect immediately below this header.
 */
package rasmus_torkel.test_framework;

public class UnitTestError extends Error
{
    public
    UnitTestError(Class<?> unitTestClass)
    {
        super(unitTestClass + " failed");
        
        // Uncaught throwables will result in a stack trace to standard error.
        // In eclipse, if we don't slow standard error down, it overtakes text previously
        // written to standard output. If we were allow this to happen, it would make
        // a horrible mess on the console. Therefore we sleep a little.
        try
        {
            Thread.sleep(100);
        }
        catch (InterruptedException e)
        {
            e.printStackTrace();
        }
    }
    
    /**
     * Uses stack trace trickery to determine class. Only invoke this constructor if the class
     * invoking it is the UnitTest subclass which fails.
     */
    public
    UnitTestError()
    {
        this(determineUnitTestClass());
    }
    
    private static Class<?>
    determineUnitTestClass()
    {
        Thread currentThread = Thread.currentThread();
        StackTraceElement[] elements = currentThread.getStackTrace();
        
        // 0 is getStackTrace,
        // 1 is this function
        // 2 is UnitTestError constructor
        // 3 is whatever calls it
        StackTraceElement element = elements[3];
        
        String className = element.getClassName();
        Class<?> clas;
        try
        {
            clas = Class.forName(className);
        }
        catch (Throwable e)
        {
            System.out.println("Can't find class for " + className);
            clas = Object.class;
        }
        return clas;
    }
}
