/*
 * Copyright Rasmus Torkel 2013
 * Author    Rasmus Torkel 2012/2013
 * 
 * You may only use this software subject to the terms and conditions which either you or your
 * organisation would have downloaded with this software and which should have been placed
 * somewhere accessible to you. These conditions are likely to
 * -- allow you or your organisation to determine whether you or your organisation
 *    may use this software for free or not and if not how to pay and how much;
 * -- prohibit or restrict republishing of this software and transmission to other organisations,
 *    even related companies within the same group;
 * -- prohibit the renaming of this class and the containing package.
 *    
 * Neither you nor your organisation may modify this header. However, you or your organisation
 * may modify anything below this header. If so, you or your organisation must place a comment
 * to that effect immediately below this header.
 */
package rasmus_torkel.text.encode;

import rasmus_torkel.set.chars.CharSet;
import rasmus_torkel.set.chars.DefinedCharSets;

/**
 * This subclass of {@link CharEncodeOptions} does not allow specification of the set of all
 * supported characters. That's because writing is in conjunction with reading, so all the
 * characters would have been checked during reading. It would not make sense to disallow
 * characters that we allow during reading and it is of no consequence if we allow characters
 * not allowed during reading, because they will not be encountered during writing anyway.
 * Therefore the set of all characters are all characters except the null character (which
 * {@link CharEncodeOptions} don't allow to be specified as legal).
 * 
 * @author Rasmus Torkel
 */
public class ReformatterCharEncodeOptions extends CharEncodeOptions
{
    public
    ReformatterCharEncodeOptions(CharEncodeOptions baseEncodeOptions)
    {
        this(baseEncodeOptions._coreChars, baseEncodeOptions._spaceEscapeOption);
    }
    
    public
    ReformatterCharEncodeOptions(CharSet           coreChars,
                                 SpaceEscapeOption spaceEscapeOption)
    {
        super(coreChars, DefinedCharSets.BITS_16_NON_NULL, spaceEscapeOption);
    }
    
    public static final ReformatterCharEncodeOptions DEFAULT =
        new ReformatterCharEncodeOptions(DefinedCharSets.ENGLISH_KEYBOARD_NON_CONTROL,
                                         CharEncodeOptions.DEFAULT._spaceEscapeOption);
}
