/*
 * Copyright Rasmus Torkel 2013
 * Author    Rasmus Torkel 2012/2013
 * 
 * You may only use this software subject to the terms and conditions which either you or your
 * organisation would have downloaded with this software and which should have been placed
 * somewhere accessible to you. These conditions are likely to
 * -- allow you or your organisation to determine whether you or your organisation
 *    may use this software for free or not and if not how to pay and how much;
 * -- prohibit or restrict republishing of this software and transmission to other organisations,
 *    even related companies within the same group;
 * -- prohibit the renaming of this class and the containing package.
 *    
 * Neither you nor your organisation may modify this header. However, you or your organisation
 * may modify anything below this header. If so, you or your organisation must place a comment
 * to that effect immediately below this header.
 */
package rasmus_torkel.text.encode;

import rasmus_torkel.misc.CharUtil;
import rasmus_torkel.misc.StringUtil;
import rasmus_torkel.set.chars.CharSet;

public class UnsupportedCharacterEncodeException extends RuntimeException
{
    public final char    _ch;
    public final int     _position;
    public final String  _textPieceDescription;
    public final String  _textPiece;
    public final String  _textSoFar;
    public final CharSet _valueCharSet;
    
    public
    UnsupportedCharacterEncodeException(char    ch,
                                        int     position,
                                        String  textPieceDescription,
                                        String  textPiece,
                                        CharSet valueCharSet)
    {
        this(ch, position, textPieceDescription, textPiece, null, valueCharSet);
    }
    
    public
    UnsupportedCharacterEncodeException(char    ch,
                                        int     position,
                                        String  textPieceDescription,
                                        String  textPiece,
                                        String  textSoFar,
                                        CharSet valueCharSet)
    {
        super("Character " + CharUtil.unicodeCharToStringForMessage(ch) + " at position " + (position + 1) +
              " of " + StringUtil.namedValueToPlainPrintablePhrase(textPieceDescription, textPiece) +
              " is not in value char set " + valueCharSet);
        _ch = ch;
        _position = position;
        _textPieceDescription = textPieceDescription;
        _textPiece = textPiece;
        _textSoFar = textSoFar;
        _valueCharSet = valueCharSet;
    }
}
