/*
 * Copyright Rasmus Torkel 2013
 * Author    Rasmus Torkel 2012/2013
 * 
 * You may only use this software subject to the terms and conditions which either you or your
 * organisation would have downloaded with this software and which should have been placed
 * somewhere accessible to you. These conditions are likely to
 * -- allow you or your organisation to determine whether you or your organisation
 *    may use this software for free or not and if not how to pay and how much;
 * -- prohibit or restrict republishing of this software and transmission to other organisations,
 *    even related companies within the same group;
 * -- prohibit the renaming of this class and the containing package.
 *    
 * Neither you nor your organisation may modify this header. However, you or your organisation
 * may modify anything below this header. If so, you or your organisation must place a comment
 * to that effect immediately below this header.
 */
package rasmus_torkel.text.read;

/**
 * A read-only text position.
 * 
 * @author Rasmus Torkel
 */
public class ROTextPosition implements Comparable<ROTextPosition>
{
    public final int _lineNr;
    public final int _charNr;
    
    public
    ROTextPosition(int lineNr,
                   int charNr)
    {
        _lineNr = lineNr;
        _charNr = charNr;
    }
    
    public
    ROTextPosition(ROTextPosition textPos)
    {
        _lineNr = textPos._lineNr;
        _charNr = textPos._charNr;
    }
    
    @Override
    public String
    toString()
    {
        return "line " + _lineNr + ", char " + _charNr;
    }

    @Override
    public int
    compareTo(ROTextPosition pos2)
    {
        if (_lineNr < pos2._lineNr)
        {
            return -1;
        }
        if (_lineNr > pos2._lineNr)
        {
            return 1;
        }
        if (_charNr < pos2._charNr)
        {
            return -1;
        }
        if (_charNr > pos2._charNr)
        {
            return 1;
        }
        return 0;
    }
}
