/*
 * Copyright Rasmus Torkel 2013
 * Author    Rasmus Torkel 2012/2013
 * 
 * You may only use this software subject to the terms and conditions which either you or your
 * organisation would have downloaded with this software and which should have been placed
 * somewhere accessible to you. These conditions are likely to
 * -- allow you or your organisation to determine whether you or your organisation
 *    may use this software for free or not and if not how to pay and how much;
 * -- prohibit or restrict republishing of this software and transmission to other organisations,
 *    even related companies within the same group;
 * -- prohibit the renaming of this class and the containing package.
 *    
 * Neither you nor your organisation may modify this header. However, you or your organisation
 * may modify anything below this header. If so, you or your organisation must place a comment
 * to that effect immediately below this header.
 */
package rasmus_torkel.text.read;

/**
 * A writable text position.
 * 
 * @author Torkel
 *
 */
public class RWTextPosition
{
    public int _lineNr = -1;
    public int _charNr = -1;
    
    /**
     * This class is intended to give ongoing access to up-to-date text positions without
     * allowing modification.
     */
    public final class View
    {
        /**
         * Gives a read-only text position.
         * @return
         */
        public ROTextPosition
        textPosition()
        {
            return new ROTextPosition(_lineNr, _charNr);
        }
        
        @Override
        public String
        toString()
        {
            ROTextPosition textPos = textPosition();
            return textPos.toString();
        }
    }
    
    public
    RWTextPosition(int lineNr,
                   int charNr)
    {
        _lineNr = lineNr;
        _charNr = charNr;
    }
    
    /**
     * Returns a read-only view of this object.
     * @return read-only view of this object
     */
    public View
    view()
    {
        return new View();
    }
    
    @Override
    public String
    toString()
    {
        return "line " + _lineNr + ", char " + _charNr;
    }
}
