/*
 * Copyright Rasmus Torkel 2013
 * Author    Rasmus Torkel 2013
 * 
 * You may only use this software subject to the terms and conditions which either you or your
 * organisation would have downloaded with this software and which should have been placed
 * somewhere accessible to you. These conditions are likely to
 * -- allow you or your organisation to determine whether you or your organisation
 *    may use this software for free or not and if not how to pay and how much;
 * -- prohibit or restrict republishing of this software and transmission to other organisations,
 *    even related companies within the same group;
 * -- prohibit the renaming of this class and the containing package.
 *    
 * Neither you nor your organisation may modify this header. However, you or your organisation
 * may modify anything below this header. If so, you or your organisation must place a comment
 * to that effect immediately below this header.
 */
package rasmus_torkel.text.type.factory.date;

import rasmus_torkel.text.type.factory.StringObjectCreationException;

public class DateValidator
{
    private static final Month[] _months =
    {
        null,
        Month.JANUARY, Month.FEBRUARY, Month.MARCH,
        Month.APRIL,   Month.MAY,      Month.JUNE,
        Month.JULY,    Month.AUGUST,   Month.SEPTEMBER,
        Month.OCTOBER, Month.NOVEMBER, Month.DECEMBER
    };
    
    public static boolean
    isLeapYear(int year)
    {
        boolean isLeapYear;
        if (year % 400 == 0)
        {
            isLeapYear = true;
        }
        else if (year % 100 == 0)
        {
            isLeapYear = false;
        }
        else if (year % 4 == 0)
        {
            isLeapYear = true;
        }
        else
        {
            isLeapYear = false;
        }
        return isLeapYear;
    }
    
    public static void
    validate(int year,
             int month,
             int day) throws StringObjectCreationException
    {
        if (month < 1 || month > 12)
        {
            throw new StringObjectCreationException(
                    "month is " + month +
                    " which is not in the range from 1 to 12");
        }
        if (day < 0)
        {
            throw new StringObjectCreationException("day is " + day + " which is less than 1");
        }
        Month monthObj = _months[month];
        boolean isLeapYear = isLeapYear(year);
        int monthLength = monthObj.length(isLeapYear);
        if (day > monthLength)
        {
            if (monthObj.isLeapYearAffected())
            {
                if (isLeapYear)
                {
                    throw new StringObjectCreationException(
                            "day is " + day +
                            " which is over the " + monthObj +
                            " leap year limit of " + monthLength);
                }
                else
                {
                    throw new StringObjectCreationException(
                            "day is " + day +
                            " which is over the " + monthObj +
                            " non-leap year limit of " + monthLength);
                }
            }
            else
            {
                throw new StringObjectCreationException(
                        "day is " + day +
                        " which is over the " + monthObj + " limit of " + monthLength);
            }
        }
    }
}
