/*
 * Copyright Rasmus Torkel 2013
 * Author    Rasmus Torkel 2013
 * 
 * You may only use this software subject to the terms and conditions which either you or your
 * organisation would have downloaded with this software and which should have been placed
 * somewhere accessible to you. These conditions are likely to
 * -- allow you or your organisation to determine whether you or your organisation
 *    may use this software for free or not and if not how to pay and how much;
 * -- prohibit or restrict republishing of this software and transmission to other organisations,
 *    even related companies within the same group;
 * -- prohibit the renaming of this class and the containing package.
 *    
 * Neither you nor your organisation may modify this header. However, you or your organisation
 * may modify anything below this header. If so, you or your organisation must place a comment
 * to that effect immediately below this header.
 */
package rasmus_torkel.text.type.factory.date;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

import rasmus_torkel.text.type.factory.StringObjectCreationException;
import rasmus_torkel.text.type.factory.StringObjectFactory;

public class DmySlashUnpaddedDateFactory implements StringObjectFactory<Date2>
{
    public static final DmySlashUnpaddedDateFactory INSTANCE =
        new DmySlashUnpaddedDateFactory();
    
    private final Pattern _pattern = Pattern.compile("(\\d+)\\/(\\d+)\\/(\\d+)");

    @Override
    public Date2
    makeObject(String objectStr)
            throws StringObjectCreationException
    {
        Matcher matcher = _pattern.matcher(objectStr);
        if (!matcher.matches())
        {
            throw new StringObjectCreationException(
                    objectStr + " does not match pattern " + _pattern.pattern());
        }
        String dayStr = matcher.group(1);
        String monthStr = matcher.group(2);
        String yearStr = matcher.group(3);
        int day = Integer.parseInt(dayStr);
        int month = Integer.parseInt(monthStr);
        int year = Integer.parseInt(yearStr);
        Date2 date;
        try
        {
            date = new Date2(year, month, day, this);
        }
        catch (StringObjectCreationException e)
        {
            throw new StringObjectCreationException(objectStr + ", " + e.getMessage(), e);
        }
        return date;
    }

    @Override
    public String makeString(Date2 date)
    {
        return date._day + "/" + date._month + "/" + date._year;
    }
    
    @Override
    public String
    toString()
    {
        return getClass().getName();
    }
}
