/*
 * Copyright Rasmus Torkel 2013
 * Author    Rasmus Torkel 2013
 * 
 * You may only use this software subject to the terms and conditions which either you or your
 * organisation would have downloaded with this software and which should have been placed
 * somewhere accessible to you. These conditions are likely to
 * -- allow you or your organisation to determine whether you or your organisation
 *    may use this software for free or not and if not how to pay and how much;
 * -- prohibit or restrict republishing of this software and transmission to other organisations,
 *    even related companies within the same group;
 * -- prohibit the renaming of this class and the containing package.
 *    
 * Neither you nor your organisation may modify this header. However, you or your organisation
 * may modify anything below this header. If so, you or your organisation must place a comment
 * to that effect immediately below this header.
 */
package rasmus_torkel.text.type.factory.date;

public enum Month
{
    JANUARY("January", 31),
    FEBRUARY("February", 28, 29),
    MARCH("March", 31),
    APRIL("April", 31),
    MAY("May", 31),
    JUNE("June", 31),
    JULY("July", 31),
    AUGUST("August", 31),
    SEPTEMBER("September", 31),
    OCTOBER("October", 31),
    NOVEMBER("November", 31),
    DECEMBER("December", 31);
    
    public final String _name;
    public final int    _length;
    public final int    _leapYearLength;
    
    private
    Month(String name,
          int    length)
    {
        this(name, length, length);
    }
    
    private
    Month(String name,
          int    length,
          int    leapYearLength)
    {
        _name = name;
        _length = length;
        _leapYearLength = leapYearLength;
    }
    
    @Override
    public String
    toString()
    {
        return _name;
    }
    
    public int
    length(boolean isLeapYear)
    {
        if (isLeapYear)
        {
            return _leapYearLength;
        }
        else
        {
            return _length;
        }
    }
    
    public boolean
    isLeapYearAffected()
    {
        return _length != _leapYearLength;
    }
}
