package rasmus_torkel.text.type.factory.date;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

import rasmus_torkel.misc.IntUtil;
import rasmus_torkel.text.type.factory.StringObjectCreationException;
import rasmus_torkel.text.type.factory.StringObjectFactory;

public class YmdHyphenPaddedDateFactory implements StringObjectFactory<Date2>
{
    public static final YmdHyphenPaddedDateFactory INSTANCE =
        new YmdHyphenPaddedDateFactory();
    
    private final Pattern _pattern = Pattern.compile("(\\d\\d\\d\\d)\\/(\\d\\d)\\/(\\d\\d)");

    @Override
    public Date2
    makeObject(String objectStr)
            throws StringObjectCreationException
    {
        Matcher matcher = _pattern.matcher(objectStr);
        if (!matcher.matches())
        {
            throw new StringObjectCreationException(
                    objectStr + " does not match pattern " + _pattern.pattern());
        }
        String yearStr = matcher.group(1);
        String monthStr = matcher.group(2);
        String dayStr = matcher.group(3);
        int day = Integer.parseInt(dayStr);
        int month = Integer.parseInt(monthStr);
        int year = Integer.parseInt(yearStr);
        Date2 date;
        try
        {
            date = new Date2(year, month, day, this);
        }
        catch (StringObjectCreationException e)
        {
            throw new StringObjectCreationException(objectStr + ", " + e.getMessage(), e);
        }
        return date;
    }

    @Override
    public String makeString(Date2 date)
    {
        StringBuilder buf = new StringBuilder(10);
        IntUtil.zeroPad(buf, date._year, 4);
        buf.append('/');
        IntUtil.zeroPad(buf, date._month, 2);
        buf.append('/');
        IntUtil.zeroPad(buf, date._day, 2);
        return buf.toString();
    }
    
    @Override
    public String
    toString()
    {
        return getClass().getName();
    }
}
