/*
 * Copyright Rasmus Torkel 2013
 * Author    Rasmus Torkel 2013
 * 
 * You may only use this software subject to the terms and conditions which either you or your
 * organisation would have downloaded with this software and which should have been placed
 * somewhere accessible to you. These conditions are likely to
 * -- allow you or your organisation to determine whether you or your organisation
 *    may use this software for free or not and if not how to pay and how much;
 * -- prohibit or restrict republishing of this software and transmission to other organisations,
 *    even related companies within the same group;
 * -- prohibit the renaming of this class and the containing package.
 *    
 * Neither you nor your organisation may modify this header. However, you or your organisation
 * may modify anything below this header. If so, you or your organisation must place a comment
 * to that effect immediately below this header.
 */
package rasmus_torkel.text.write;

/**
 * <p>
 * Represents the various supported line endings.
 * </p>
 * <p>
 * Each type is named after the most popular operating system category where it is
 * standard. According to my research, the explicit types offered here are the only
 * ones in use by modern operating systems.
 * </p>
 * 
 * @author Rasmus Torkel
 *
 */
public enum LineEndType
{
    UNIX("\n", "UNIX"),
    WINDOWS("\r\n", "Windows"),
    MAC("\r", "Mac");
    
    public final String _lineEndStr;
    public final String _label;
    
    private
    LineEndType(String lineEndStr,
                String label)
    {
        _lineEndStr = lineEndStr;
        _label = label;
    }
    
    @Override
    public String
    toString()
    {
        // Returns _lineEndStr and not _label because this class is more about the
        // actual line end rather than the choice of line end.
        return _lineEndStr;
    }
}
