/*
 * Copyright Rasmus Torkel 2012
 * Author    Rasmus Torkel 2012
 * 
 * You may only use this software subject to the terms and conditions which either you or your
 * organisation would have downloaded with this software and which should have been placed
 * somewhere accessible to you. These conditions are likely to
 * -- allow you or your organisation to determine whether you or your organisation
 *    may use this software for free or not and if not how to pay and how much;
 * -- prohibit or restrict republishing of this software and transmission to other organisations,
 *    even related companies within the same group;
 * -- prohibit the renaming of this class and the containing package.
 *    
 * Neither you nor your organisation may modify this header. However, you or your organisation
 * may modify anything below this header. If so, you or your organisation must place a comment
 * to that effect immediately below this header.
 */
package rasmus_torkel.xml_basic.read.exception;

import rasmus_torkel.set.chars.CharSet;
import rasmus_torkel.text.read.ROTextPosition;
import rasmus_torkel.text.read.exception.CharSetException;

/**
 * Indicates that an attribute value or text node contains a character that
 * is contrary to the char set in the read options.
 * <p>
 * In traditional XML parsing, invalid characters would more naturally be a syntax
 * problem. However while in traditional XML processing the set of supported characters
 * depends on the XML version, this parser does not consider any character inherently
 * invalid. That's why this exception is in the semantic exception hierarchy.
 * 
 * @author Rasmus Torkel
 *
 */
public class XmlCharSetException extends CharSetException
{
    public
    XmlCharSetException(String         xmlContext,
                        char           ch,
                        CharSet        charSet,
                        ROTextPosition textPos)
    {
        this(xmlContext, makeMessageCore(ch, charSet, textPos));
    }
    
    public
    XmlCharSetException(String         xmlContext,
                        String         xmlStringLabel,
                        String         xmlString,
                        char           ch,
                        int            chPosInXmlString,
                        CharSet        charSet,
                        ROTextPosition textPos)
    {
        this(xmlContext, makeMessageCore(xmlStringLabel, xmlString, ch, chPosInXmlString, charSet, textPos));
    }
    
    private static String
    makeMessageCore(char           ch,
                    CharSet        charSet,
                    ROTextPosition textPos)
    {
        return "Character with hex value " + Integer.toString((int)ch, 16) +
               " at position " + textPos + " is not in CharSet " + charSet;
    }
    
    public
    XmlCharSetException(String xmlContext,
                        String messageCore)
    {
        super(xmlContext, messageCore);
    }
    
    private static String
    makeMessageCore(String         xmlStringLabel,
                    String         xmlString,
                    char           ch,
                    int            chPosInXmlString,
                    CharSet        charSet,
                    ROTextPosition textPos)
    {
        return xmlStringLabel + " " + xmlString + " has character with hex value " +
               Integer.toString((int)ch, 16) + " at position " + chPosInXmlString +
               " (near position " + textPos + " in XML Input) which is not in CharSet " + charSet;
    }
}
