/*
 * Copyright Rasmus Torkel 2012
 * Author    Rasmus Torkel 2012
 * 
 * You may only use this software subject to the terms and conditions which either you or your
 * organisation would have downloaded with this software and which should have been placed
 * somewhere accessible to you. These conditions are likely to
 * -- allow you or your organisation to determine whether you or your organisation
 *    may use this software for free or not and if not how to pay and how much;
 * -- prohibit or restrict republishing of this software and transmission to other organisations,
 *    even related companies within the same group;
 * -- prohibit the renaming of this class and the containing package.
 *    
 * Neither you nor your organisation may modify this header. However, you or your organisation
 * may modify anything below this header. If so, you or your organisation must place a comment
 * to that effect immediately below this header.
 */
package rasmus_torkel.xml_basic.read.exception;

import rasmus_torkel.text.read.ROTextPosition;
import rasmus_torkel.text.read.exception.ReadOptionsEnforcementException;
import rasmus_torkel.xml_basic.read.TagNode;
import rasmus_torkel.xml_basic.read.TextNode;

public class XmlTextAndChildException extends ReadOptionsEnforcementException
{
    public
    XmlTextAndChildException(String         context,
                             TagNode        containingTagNode,
                             ROTextPosition textPos,
                             TagNode        childNode)
    {
        super(context, makeMessageCore(containingTagNode, textPos, childNode));
    }
    
    public
    XmlTextAndChildException(String         context,
                             TagNode        containingTagNode,
                             ROTextPosition childPos,
                             TextNode       textNode)
    {
        super(context, makeMessageCore(containingTagNode, childPos, textNode));
    }
    
    private static String
    makeMessageCore(TagNode        containingTagNode,
                    ROTextPosition textPos,
                    TagNode        childNode)
    {
        return containingTagNode + " contains both direct non-whitespace text at " +
               textPos + " and child node at " + childNode._textPos +
               " in contravention of the read options";
    }
    
    private static String
    makeMessageCore(TagNode        containingTagNode,
                    ROTextPosition childPos,
                    TextNode       textNode)
    {
        return containingTagNode + " contains both child node at " + childPos +
               " and direct non-whitespace text at " + textNode._textPos +
               " in contravention of the read options";
    }
}
