package rasmus_torkel.xml_basic.read.factory;

import rasmus_torkel.text.read.exception.WrongNodeException;
import rasmus_torkel.xml_basic.read.TagNode;
import rasmus_torkel.xml_basic.rw.TagNodeId;
import rasmus_torkel.xml_basic.rw.XmlNameSpace;
import rasmus_torkel.xml_basic.write.XmlSink;
import rasmus_torkel.xml_basic.write.XmlSinkWritable;

public class ChoiceXmlObjectFactory<T> extends XmlObjectFactory<T> implements XmlSinkWritable
{
    public static class Option implements XmlSinkWritable
    {
        public final TagNodeId           _id;
        public final XmlObjectFactory<?> _factory;
        
        public
        Option(TagNodeId           id,
               XmlObjectFactory<?> factory)
        {
            _id = id;
            _factory = factory;
        }
        
        public
        Option(MainXmlObjectFactory<?> factory)
        {
            this(factory._naturalId, factory);
        }

        @Override
        public TagNodeId
        naturalTagNodeId()
        {
            return new TagNodeId("option");
        }

        @Override
        public void
        mostToXml(XmlSink      xmlSink,
                  String       relativeName,
                  XmlNameSpace nameSpace)
        {
            xmlSink.sinkSimpleNode("id", _id);
            xmlSink.sinkNode("factory", (XmlSinkWritable)_factory);
        }
    }
    
    private final Option[] _options;
    
    public
    ChoiceXmlObjectFactory(Class<T> supportedClass,
                           Option[] options)
    {
        super(supportedClass);
        _options = options;
        for (Option option : options)
        {
            if (!_supportedClass.isAssignableFrom(option._factory._supportedClass))
            {
                throw new RuntimeException(
                        "ChoiceXmlObjectFactory support " + _supportedClass +
                        " which can't be assigned from " + option._factory._supportedClass +
                        " supported by option with id " + option._id);
            }
        }
    }
    
    public
    ChoiceXmlObjectFactory(Class<T>                  supportedClass,
                           MainXmlObjectFactory<?>[] factories)
    {
        this(supportedClass, mainFactoriesToOptions(factories));
    }
    
    private static Option[]
    mainFactoriesToOptions(MainXmlObjectFactory<?>[] factories)
    {
        Option[] options = new Option[factories.length];
        for (int i = 0; i < options.length; i++)
        {
            options[i] = new Option(factories[i]);
        }
        return options;
    }

    @Override
    public T
    extractFromNode(TagNode node)
    {
        throw new RuntimeException("This function should not have been called");
    }

    @SuppressWarnings("unchecked")
    @Override
    public T
    nextOnParentN(TagNode      parentNode,
                  String       relativeName,
                  XmlNameSpace nameSpace)
    {
        TagNode node = nextChildNodeN(parentNode, relativeName, nameSpace);
        if (node == null)
        {
            return null;
        }
        for (Option option : _options)
        {
            if (option._id.equals(node._id))
            {
                T element = (T)option._factory.extractFromNode(node);
                if (element == null)
                {
                    throw new RuntimeException(
                            "extractFromNode returned null for factory of type " +
                            option._factory.getClass().getName());
                }
                node.verifyNoMoreChildren();
                return element;
            }
        }
        throw new WrongNodeException(
                node._xmlContext,
                node + " is not a supported option for choice factory class " + _supportedClass.getName());
    }

    @Override
    public TagNodeId
    naturalTagNodeId()
    {
        return new TagNodeId("choiceFactory");
    }

    @Override
    public void
    mostToXml(XmlSink      xmlSink,
              String       relativeName,
              XmlNameSpace nameSpace)
    {
        super.mostToXml(xmlSink, relativeName, nameSpace);
        xmlSink.sinkArrayNode("options", _options);
    }
}
