/*
 * Copyright Rasmus Torkel 2013
 * Author    Rasmus Torkel 2012/2013
 * 
 * You may only use this software subject to the terms and conditions which either you or your
 * organisation would have downloaded with this software and which should have been placed
 * somewhere accessible to you. These conditions are likely to
 * -- allow you or your organisation to determine whether you or your organisation
 *    may use this software for free or not and if not how to pay and how much;
 * -- prohibit or restrict republishing of this software and transmission to other organisations,
 *    even related companies within the same group;
 * -- prohibit the renaming of this class and the containing package.
 *    
 * Neither you nor your organisation may modify this header. However, you or your organisation
 * may modify anything below this header. If so, you or your organisation must place a comment
 * to that effect immediately below this header.
 */
package rasmus_torkel.xml_basic.rw;

/**
 * Represents the tag at the beginning (and often the end) of a tag node.
 * 
 * @author Torkel
 *
 */
public class TagNodeId
{
    /**
     * Relative name. In other words the name less anything to do with name spaces.
     * Of course, if the node is not in a name space, that's just the whole tag.
     */
    public final String       _relativeName;
    
    /**
     * Name space that the node is in, if any. Null if there is no name space.
     */
    public final XmlNameSpace _nameSpace;
    
    public
    TagNodeId(String       relativeName)
    {
        this(relativeName, null);
    }
    
    public
    TagNodeId(String       relativeName,
              XmlNameSpace nameSpace)
    {
        _relativeName = relativeName;
        _nameSpace = nameSpace;
        if (_relativeName == null)
        {
            throw new RuntimeException("null relative name for TagNodeId");
        }
    }
    
    public String
    displayName()
    {
        if (_nameSpace == null || _nameSpace._prefix == null)
        {
            return _relativeName;
        }
        else
        {
            return _nameSpace._prefix + ":" + _relativeName;
        }
    }
    
    public String
    fullName()
    {
        if (_nameSpace == null || _nameSpace._prefix == null)
        {
            return _relativeName;
        }
        else
        {
            return _nameSpace._name + ":" + _relativeName;
        }
    }
    
    public String
    comparisonName()
    {
        return makeComparisonName(_relativeName, _nameSpace);
    }
    
    public static String
    makeComparisonName(String       relativeName,
                       XmlNameSpace nameSpace)
    {
        if (nameSpace == null)
        {
            return relativeName + " without name space";
        }
        else
        {
            return relativeName + " in name space " + nameSpace._name;
        } 
    }
    
    public boolean
    isSame(String       relativeName,
           XmlNameSpace nameSpace)
    {
        if (!_relativeName.equals(relativeName))
        {
            return false;
        }
        return XmlNameSpace.isSame(nameSpace, _nameSpace);
    }
    
    @Override
    public String
    toString()
    {
        return fullName();
    }
    
    @Override
    public boolean
    equals(Object other)
    {
        if (other == null)
        {
            return false;
        }
        if (!(other instanceof TagNodeId))
        {
            return false;
        }
        TagNodeId otherId = (TagNodeId)other;
        return isSame(otherId._relativeName, otherId._nameSpace);
    }
}
