package rasmus_torkel.xml_basic.rw;

import rasmus_torkel.misc.CharUtil;
import rasmus_torkel.set.chars.CharSet;
import rasmus_torkel.set.chars.ComboCharSet;
import rasmus_torkel.set.chars.RangeCharSet;
import rasmus_torkel.set.strings.JustifyingStringSet;

public class XmlChars
{
    public static final JustifyingStringSet NAME_SET = new JustifyingStringSet()
    {
        @Override
        public boolean
        isIn(String s)
        {
            return outReason(s) == null;
        }
        
        @Override
        public String
        outReason(String s)
        {
            if (s == null)
            {
                return "is null";
            }
            int len = s.length();
            if (len == 0)
            {
                return "is empty";
            }
            {
                char ch0 = s.charAt(0);
                if (!isNameStartChar(ch0))
                {
                    return "starts with " + CharUtil.unicodeCharToStringForMessage(ch0) +
                           " which is not a legal name starter";
                }
            }
            for (int i = 1; i < len; i++)
            {
                char ch = s.charAt(i);
                if (!isNameChar(ch))
                {
                    return "has character " + CharUtil.unicodeCharToStringForMessage(ch) +
                           " in position " + (i + 1) + " which is not a legal name char";
                }
            }
            return null;
        }
    };
    
    public static final CharSet VALUE_CHAR_SET =
            new ComboCharSet("XML_Text",
                             new RangeCharSet((char)0x1, (char)0xD7FF, "XML_Text_Part_1"),
                             new RangeCharSet((char)0xE000, (char)0xFFFD, "XML_Text_Part_2"));
    
    public static boolean
    isNameStartChar(char ch)
    {
        int chi = (int)ch;
        if (   'A' <= ch && ch <= 'Z'
            || ch == '_'
            || 'a' <= ch && ch <= 'z'
            || 0xC0 <= chi && chi <= 0xD6
            || 0xD8 <= chi && chi <= 0xF6
            || 0xF8 <= chi && chi <= 0x2FF
            || 0x370 <= chi && chi <= 0x37D
            || 0x37F <= chi && chi <= 0x1FFF
            || 0x200C <= chi && chi <= 0x200D
            || 0x2070 <= chi && chi <= 0x218F
            || 0x2C00 <= chi && chi <= 0x2FEF
            || 0x3001 <= chi && chi <= 0xD7FF
            || 0xF900 <= chi && chi <= 0xFDCF
            || 0xFDF0 <= chi && chi <= 0xFFFD
            || 0x10000 <= chi && chi <= 0xEFFFF)
        {
            return true;
        }
        else
        {
            return false;
        }
    }
    
    public static boolean
    isNameChar(char ch)
    {
        int chi = (int)ch;
        if (isNameStartChar(ch))
        {
            return true;
        }
        else if (   ch == '-'
                 || ch == '.'
                 || '0' <= ch && ch <= '9'
                 || chi == 0xB7
                 || 0x0300 <= chi && chi <= 0x036F
                 || 0x203F <= chi && chi <= 0x2040)
        {
            return true;
        }
        else
        {
            return false;
        }
    }
}
