/*
 * Copyright Rasmus Torkel 2016
 * Author    Rasmus Torkel 2016
 * 
 * You may only use this software subject to the terms and conditions which either you or your
 * organisation would have downloaded with this software and which should have been placed
 * somewhere accessible to you. These conditions are likely to
 * -- allow you or your organisation to determine whether you or your organisation
 *    may use this software for free or not and if not how to pay and how much;
 * -- prohibit or restrict republishing of this software and transmission to other organisations,
 *    even related companies within the same group;
 * -- prohibit the renaming of this class and the containing package.
 *    
 * Neither you nor your organisation may modify this header. However, you or your organisation
 * may modify anything below this header. If so, you or your organisation must place a comment
 * to that effect immediately below this header.
 */
package rasmus_torkel.xml_basic.tool;

import java.io.File;

import rasmus_torkel.xml_basic.read.TagNode;
import rasmus_torkel.xml_basic.read.XmlReadOptions;
import rasmus_torkel.xml_basic.read.impl.XmlReader;

/**
 * Tool for validating XML files. Can be used as tool with main function
 * or as library.
 * 
 * @author Rasmus Torkel
 */
public class XmlValidator
{
    /**
     * Validates the files specified in the command line.
     * @param args Names of files to be validated.
     */
    public static void
    main(String[] args)
    {
        System.out.println("Quantity of files to be validated: " + args.length);
        for (String fileName : args)
        {
            File xmlFile = new File(fileName);
            Throwable t = validate(xmlFile);
            if (t == null)
            {
                System.out.println(fileName + " is valid");
            }
            else
            {
                System.out.println(fileName + " is not a valid XML file, " + t);
            }
        }
    }


    /**
     * Reads through an XML file.
     * 
     * @param xmlFile     To be validated
     * @return null if validation succeeds, Throwable otherwise
     */
    public static Throwable
    validate(File xmlFile)
    {
        return validate(xmlFile, XmlReadOptions.DEFAULT);
    }

    /**
     * Reads through an XML file.
     * 
     * @param xmlFile     To be validated
     * @param readOptions For reading. Will make a difference to what is
     *                    considered valid and what isn't
     * @return null if validation succeeds, Throwable otherwise
     */
    public static Throwable
    validate(File           xmlFile,
             XmlReadOptions readOptions)
    {
        try
        {
            read(xmlFile, readOptions);
            return null;
        }
        catch (Throwable t)
        {
            return t;
        }
    }
    
    /**
     * Reads through an XML file, throwing an exception if it isn't valid.
     * 
     * @param xmlFile     To be validated
     * @param readOptions For reading. Will make a difference to what is
     *                    considered valid and what isn't
     */
    public static void
    read(File           xmlFile,
         XmlReadOptions readOptions)
    {
        TagNode root = XmlReader.xmlFileToRoot(xmlFile, readOptions);
        try
        {
            root.readRest();
        }
        finally
        {
            root.closeReader();
        }
    }
}
