/*
 * Copyright Rasmus Torkel 2013
 * Author    Rasmus Torkel 2013
 * 
 * You may only use this software subject to the terms and conditions which either you or your
 * organisation would have downloaded with this software and which should have been placed
 * somewhere accessible to you. These conditions are likely to
 * -- allow you or your organisation to determine whether you or your organisation
 *    may use this software for free or not and if not how to pay and how much;
 * -- prohibit or restrict republishing of this software and transmission to other organisations,
 *    even related companies within the same group;
 * -- prohibit the renaming of this class and the containing package.
 *    
 * Neither you nor your organisation may modify this header. However, you or your organisation
 * may modify anything below this header. If so, you or your organisation must place a comment
 * to that effect immediately below this header.
 */
package rasmus_torkel.xml_basic.types;

import rasmus_torkel.text.type.factory.StringObjectCreationException;
import rasmus_torkel.text.type.factory.StringObjectFactory;
import rasmus_torkel.xml_basic.read.TagNode;
import rasmus_torkel.xml_basic.rw.TagNodeId;
import rasmus_torkel.xml_basic.rw.XmlNameSpace;

public class StringObjectXmlFactory<T>
{
    private final StringObjectFactory<T> _coreFactory;
    
    public
    StringObjectXmlFactory(StringObjectFactory<T> coreFactory)
    {
        _coreFactory = coreFactory;
    }
    
    public T
    nextFieldN(TagNode      parent,
               String       relativeName,
               XmlNameSpace nameSpace)
    {
        TagNode child = parent.nextChildN(relativeName, nameSpace);
        if (child == null)
        {
            return null;
        }
        return extractFromChild(child);
    }
    
    public T
    nextFieldE(TagNode      parent,
               String       relativeName,
               XmlNameSpace nameSpace)
    {
        TagNode child = parent.nextChildE(relativeName, nameSpace);
        return extractFromChild(child);
    }
    
    public T
    nextFieldD(TagNode      parent,
               String       relativeName,
               XmlNameSpace nameSpace,
               T            defaultValue)
    {
        TagNode child = parent.nextChildN(relativeName, nameSpace);
        if (child == null)
        {
            return defaultValue;
        }
        return extractFromChild(child);
    }
    
    public T
    nextFieldN(TagNode      parent,
               String       simpleName)
    {
        return nextFieldN(parent, simpleName, null);
    }
    
    public T
    nextFieldE(TagNode      parent,
               String       simpleName)
    {
        return nextFieldE(parent, simpleName, null);
    }
    
    public T
    nextFieldD(TagNode      parent,
               String       simpleName,
               T            defaultValue)
    {
        return nextFieldD(parent, simpleName, null, defaultValue);
    }
    
    public T
    nextFieldN(TagNode      parent,
               TagNodeId    id)
    {
        return nextFieldN(parent, id._relativeName, id._nameSpace);
    }
    
    public T
    nextFieldE(TagNode      parent,
               TagNodeId    id)
    {
        return nextFieldE(parent, id._relativeName, id._nameSpace);
    }
    
    public T
    nextFieldD(TagNode      parent,
               TagNodeId    id,
               T            defaultValue)
    {
        return nextFieldD(parent, id._relativeName, id._nameSpace, defaultValue);
    }

    protected T
    extractFromChild(TagNode child)
    {
        String textField = child.onlyText();
        try
        {
            T t = _coreFactory.makeObject(textField);
            return t;
        }
        catch (StringObjectCreationException e)
        {
            throw new StringObjectCreationXmlException(
                    child._xmlContext,
                    child + " contains text " + textField +
                    " which can not be converted by " + _coreFactory.toString() +
                    ", " + e.getMessage());
        }
    }
}
