/*
 * Copyright Rasmus Torkel 2013
 * Author    Rasmus Torkel 2013
 * 
 * You may only use this software subject to the terms and conditions which either you or your
 * organisation would have downloaded with this software and which should have been placed
 * somewhere accessible to you. These conditions are likely to
 * -- allow you or your organisation to determine whether you or your organisation
 *    may use this software for free or not and if not how to pay and how much;
 * -- prohibit or restrict republishing of this software and transmission to other organisations,
 *    even related companies within the same group;
 * -- prohibit the renaming of this class and the containing package.
 *    
 * Neither you nor your organisation may modify this header. However, you or your organisation
 * may modify anything below this header. If so, you or your organisation must place a comment
 * to that effect immediately below this header.
 */
package rasmus_torkel.xml_basic.write;


public class AttributeLineOptions
{
    public final boolean _alwaysSplitRootTag;
    public final boolean _alwaysSplitXmlnsTag;
    public final int     _attributeQtySplitThreshold;
    
    public static final int NO_ATTRIBUTE_QTY_SPLIT_THRESHOLD = Integer.MAX_VALUE;
    
    public static final AttributeLineOptions DEFAULT = new AttributeLineOptions(false, true, 2);
    public static final AttributeLineOptions NO_SPLIT = new AttributeLineOptions(false, false);
    public static final AttributeLineOptions SPLIT = new AttributeLineOptions(true, true, 1);
    
    public
    AttributeLineOptions(boolean alwaysSplitRootTag,
                         boolean alwaysSplitXmlnsTag)
    {
        this(alwaysSplitRootTag, alwaysSplitXmlnsTag, NO_ATTRIBUTE_QTY_SPLIT_THRESHOLD);
    }
    
    public
    AttributeLineOptions(boolean alwaysSplitRootTag,
                         boolean alwaysSplitXmlnsTag,
                         int     attributeQtySplitThreshold)
    {
        _alwaysSplitRootTag = alwaysSplitRootTag;
        _alwaysSplitXmlnsTag = alwaysSplitXmlnsTag;
        _attributeQtySplitThreshold = attributeQtySplitThreshold;
    }
    
    @Override
    public String
    toString()
    {
        return "{ alwaysSplitRootTag = " + _alwaysSplitRootTag +
               ", alwaysSplitXmlnsTag = " + _alwaysSplitXmlnsTag +
               ", attributeQtySplitThreshold = " + _attributeQtySplitThreshold + " }";
    }
    
    public void
    toXml(XmlSink xmlSink,
          String  fieldName)
    {
        xmlSink.startNode(fieldName);
        xmlSink.sinkAttribute("type", getClass().getName());
        xmlSink.sinkSimpleNode("alwaysSplitRootTag", _alwaysSplitRootTag);
        xmlSink.sinkSimpleNode("alwaysSplitXmlnsTag", _alwaysSplitXmlnsTag);
        xmlSink.sinkSimpleNode("attributeQtySplitThreshold", _attributeQtySplitThreshold);
        xmlSink.closeNode();
    }
}
