/*
 * Copyright Rasmus Torkel 2014
 * Author    Rasmus Torkel 2014
 * 
 * You may only use this software subject to the terms and conditions which either you or your
 * organisation would have downloaded with this software and which should have been placed
 * somewhere accessible to you. These conditions are likely to
 * -- allow you or your organisation to determine whether you or your organisation
 *    may use this software for free or not and if not how to pay and how much;
 * -- prohibit or restrict republishing of this software and transmission to other organisations,
 *    even related companies within the same group;
 * -- prohibit the renaming of this class and the containing package.
 *    
 * Neither you nor your organisation may modify this header. However, you or your organisation
 * may modify anything below this header. If so, you or your organisation must place a comment
 * to that effect immediately below this header.
 */
package rasmus_torkel.xml_basic.write;

import rasmus_torkel.misc.ArrayMaker;
import rasmus_torkel.xml_basic.rw.TagNodeId;
import rasmus_torkel.xml_basic.rw.XmlNameSpace;

/**
 * Interface for objects that can convert themselves to XML by writing themselves to
 * an {@link XmlSink}.
 * 
 * @author Rasmus Torkel
 */
public interface XmlSinkWritable
{
    public static ArrayMaker<XmlSinkWritable> ARRAY_MAKER = new ArrayMaker<XmlSinkWritable>()
    {
        @Override
        public XmlSinkWritable[]
        newArray(int size)
        {
            return new XmlSinkWritable[size];
        }
    };
    
    /**
     * When writing instances of this class, the id returned
     * by this function will be used unless alternative
     * identification is supplied.
     * @return The identifier to use for the node  if no other identification is supplied.
     */
    public TagNodeId
    naturalTagNodeId();
    
    /**
     * Causes instance to write itself to XML except that it doesn't
     * start the node nor writes the closing tag.
     * @param xmlSink      Where the XML is written
     * @param relativeName or simple node name. May not be needed but is available.
     * @param nameSpace    or null for simple node names. May not be needed but is available.
     */
    public void
    mostToXml(XmlSink      xmlSink,
              String       relativeName,
              XmlNameSpace nameSpace);
}
