/*
 * Copyright Rasmus Torkel 2013
 * Author    Rasmus Torkel 2013
 * 
 * You may only use this software subject to the terms and conditions which either you or your
 * organisation would have downloaded with this software and which should have been placed
 * somewhere accessible to you. These conditions are likely to
 * -- allow you or your organisation to determine whether you or your organisation
 *    may use this software for free or not and if not how to pay and how much;
 * -- prohibit or restrict republishing of this software and transmission to other organisations,
 *    even related companies within the same group;
 * -- prohibit the renaming of this class and the containing package.
 *    
 * Neither you nor your organisation may modify this header. However, you or your organisation
 * may modify anything below this header. If so, you or your organisation must place a comment
 * to that effect immediately below this header.
 */
package rasmus_torkel.xml_basic.write;

import rasmus_torkel.text.encode.CharEncodeOptions;
import rasmus_torkel.text.encode.CharMaplet;
import rasmus_torkel.text.encode.TextEncoder;

public class XmlTextEncoder extends TextEncoder
{
    private static final CharMaplet AMP = new CharMaplet('&', "&amp;");
    private static final CharMaplet LT = new CharMaplet('<', "&lt;");
    private static final CharMaplet QUOT = new CharMaplet('"', "&quot;");
    private static final CharMaplet APOS = new CharMaplet('\'', "&apos;");
    
    private static final CharMaplet[] TEXT_NODE_MAPLETS = {AMP, LT};
    private static final CharMaplet[] SINGLE_QUOTED_MAPLETS = {AMP, LT, APOS};
    private static final CharMaplet[] DOUBLE_QUOTED_MAPLETS = {AMP, LT, QUOT};
    
    public
    XmlTextEncoder(CharEncodeOptions encodeOptions,
                   CharMaplet[]      maplets)
    {
        super(encodeOptions, maplets);
    }
    
    public
    XmlTextEncoder(CharMaplet[]      maplets)
    {
        super(CharEncodeOptions.DEFAULT, maplets);
    }
    
    public static XmlTextEncoder
    makeTextNodeEncoder(CharEncodeOptions encodeOptions)
    {
        return new XmlTextEncoder(encodeOptions, TEXT_NODE_MAPLETS);
    }
    
    public static XmlTextEncoder
    makeSingleQuotedEncoder(CharEncodeOptions encodeOptions)
    {
        return new XmlTextEncoder(encodeOptions, SINGLE_QUOTED_MAPLETS);
    }
    
    public static XmlTextEncoder
    makeDoubleQuotedEncoder(CharEncodeOptions encodeOptions)
    {
        return new XmlTextEncoder(encodeOptions, DOUBLE_QUOTED_MAPLETS);
    }
    
    protected void
    encodeUniCodeChar(StringBuilder buf,
                      char          ch)
    {
        buf.append("&#x");
        charToHex(buf, ch);
        buf.append(';');
    }

    @Override
    protected void
    encodeWhiteSpaceChar(StringBuilder buf,
                         char          ch)
    {
        encodeUniCodeChar(buf, ch);
    }
}
